var Class    = require("core").Class;
var Request  = require("core").Net.Request;
var Response = require("core").Net.Response;
var Server   = require("core").Net.Server;
var YAML     = require("core").Yaml.YAML;
var JsonRpc  = require("core").JsonRpc;

var Fs = require('fs');

var RenderMethod = require("./RenderMethod.js").RenderMethod;
var QueryMethod = require("./QueryMethod.js").QueryMethod;
var ResourceMethod = require("./ResourceMethod.js").ResourceMethod;

var DevRouter = require("./router/DevRouter.js").DevRouter;
var ResourceRouter = require("./router/ResourceRouter.js").ResourceRouter;
var StaticRouter = require("./router/StaticRouter.js").StaticRouter;

var Cache = require("./cache/Cache.js").Cache;

var TemplateApiManager = require("./api/TemplateApiManager.js").TemplateApiManager;


/**
 * @class Server Main http server class. Wraps native node.js http object. Creates Request processor instance and dispatch processing. 
 * @param {Class} RequestProcessorClass Subclass of RequestProcessor family.
 */
var TemplateServer = function(){
    this.Extends = JsonRpc.JsonRpcServer;
    
    this._cache = null;
    this._templatePath = null;
    //dla dev controllera
    this._actionsPath = null;
    this._staticPath = null;
    this._resourcePath = null;
    this.router = null;
    this.resourceRouter = null;
    this.staticRouter = null;
    this._plugins = null;
    
    this.api = null;
    
    this.initialize = function(){
        //preparing stuff
        try {
           this.configuration = YAML.parseFileSync("./config/conf.yaml").pop();
           this._templatePath   = this.configuration.hasOwnProperty('templatePath') ? this.configuration.templatePath : 'templates/';
           this._actionsPath    = this.configuration.hasOwnProperty('actionsPath') ? this.configuration.actionsPath : 'actions/';
           this._staticPath     = this.configuration.hasOwnProperty('staticPath') ? this.configuration.staticPath : 'static/';
           this._resourcePath   = this.configuration.hasOwnProperty('resourcePath') ? this.configuration.resourcePath : 'dynamic/';
           this._resourcesConfigPath = this.configuration.hasOwnProperty('resourcesConfigPath') ? this.configuration.resourcesPath : 'dynamic/resources.yaml';
        } catch(e){
           console.log('Configuration file (./config/conf.yaml) does not exists or is invalid. Using default configuration...')
           this._templatePath = "templates/";
           this._actionsPath = "actions/"
           this._staticPath = "static/"
           this._resourcePath = "dynamic";
           this._resourcesConfigPath = 'dynamic/resources.yaml';
        }
        
        console.log("\n---\n| Template CACHEing:");
        this._cache = new Cache([
            {
                className: "JqTplCache",
                resourcePath: this._templatePath,
                excludePattern: ".svn"
            }
        ]);
        
        try{
          this._cache.cache();
        } catch(e){
          console.log('  - ERROR: ' + e.message);
          process.exit(1);
        }
        
        
        //Kontroler -> Router
        try {
            this.router = new DevRouter(this._actionsPath);
            this.resourceRouter = new ResourceRouter(this._resourcePath);
            this.staticRouter = new StaticRouter(this._staticPath);
        }
        catch(e){
            console.log("----", e.message, e.stack);
        }
        // create and prepare template api objects
        this._createApi(this.configuration.api);
        
        // create 
        this._createPlugins();
        
        // starting server
        this.parent();
        this.addMethod('render', RenderMethod);
        this.addMethod('query', QueryMethod);
        this.addMethod('resource', ResourceMethod);
    };
    
    this._createPlugins = function(){
        var fs = require('fs');
        var resource = YAML.parseFileSync(this._resourcesConfigPath).pop();
        var config = {};
        for(var file in resource.files){
            var conf = resource.files[file];
            config[file] = conf;
            config[file]['class'] = resource.classes[config[file]['class']];
            var files = {};
            for(var i=0,max=config[file].files.length;i<max;i++){
                files[config[file].files[i]] = fs.readFileSync('./css/'+config[file].files[i], 'utf8') + "\n\n";
            }
            config[file].files = files;
        }
        this._plugins = config;
    };
    
    this.getPluginsConfig = function(){
        return this._plugins;
    };
    
    
    this._createApi = function(configuration){
        console.log("\n---\n| Template API init:")
        var apiManager = new TemplateApiManager();
        apiManager.addEventListener(TemplateApiManager.Event.READY, function(e){
            console.log("  - All API's are READY!\n");
        });
        apiManager.addEventListener(TemplateApiManager.Event.ERROR, function(e){
            console.log('  - API: ERROR: ' + e.message);
            process.exit(1);
        });
        this.api = {
            ocdn : new (require("./api/Ocdn.js").Ocdn)(configuration["ocdn"].configuration)
        };
        for(var prop in this.api){
            console.log("  - '" + prop + "' api created");
            apiManager.add(this.api[prop]);
        }
        apiManager.start();
    };

    
    /**
     * Returns a server template path
     * 
     * @returns {String}
     */
    this.getTemplatePath = function(){
        return this._templatePath;
    };

    /**
     * Sets server template path
     * 
     * @param {string} templatePath Path to templates folder.
     */
    this.setTemplatePath = function(templatePath){
        this._templatePath = templatePath;
    };
}
TemplateServer.Exception = {};
TemplateServer = new Class(new TemplateServer());
exports.TemplateServer = TemplateServer;
